package gov.va.med.foundations.samples;

import gov.va.med.foundations.adapter.cci.VistaLinkConnection;
import gov.va.med.foundations.adapter.cci.VistaLinkServerInfo;
import gov.va.med.foundations.adapter.record.VistaLinkFaultException;
import gov.va.med.foundations.rpc.RpcRequest;
import gov.va.med.foundations.rpc.RpcRequestFactory;
import gov.va.med.foundations.rpc.RpcResponse;
import gov.va.med.foundations.rpc.RpcReferenceType;
import gov.va.med.foundations.security.vistalink.CallbackHandlerUnitTest;
import gov.va.med.foundations.security.vistalink.VistaLoginModuleException;
import gov.va.med.foundations.security.vistalink.VistaKernelPrincipalImpl;
import gov.va.med.foundations.security.vistalink.VistaLoginModuleLoginsDisabledException;
import gov.va.med.foundations.security.vistalink.VistaLoginModuleNoJobSlotsAvailableException;
import gov.va.med.foundations.security.vistalink.VistaLoginModuleNoPathToListenerException;
import gov.va.med.foundations.security.vistalink.VistaLoginModuleTooManyInvalidAttemptsException;
import gov.va.med.foundations.security.vistalink.VistaLoginModuleUserCancelledException;
import gov.va.med.foundations.security.vistalink.VistaLoginModuleUserTimedOutException;
import gov.va.med.foundations.utilities.ExceptionUtils;
import gov.va.med.foundations.utilities.FoundationsException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.StringTokenizer;
import java.util.TreeMap;

import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;

/**
 * Console application demonstration of VistaLink's Vista connectivity and RPC functionality.
 * <p><b>Command-Line Parameters</b><p>This application uses default settings to connect to a Vista server. To override
 * the default setting use any of the following commmand-line flags:
 * <ul>
 * <li>-s server appname to use from the JAAS config file
 * <li>-a access code
 * <li>-v verify code
 * <li>-d division ien
 * <li>-i number of iterations
 * <li>-p {true|false} use proprietary message format instead of XML
 * </ul> Also, a JAAS configuration file must be specified. The easiest way is to specify the location/name of
 * the JAAS configuration file with the -Djava.security.auth.login.config VM argument.
 * <p>For example:
 * <pre>
 * java -Djava.security.auth.login.config=./jaas.config VistaLinkRpcConsole -s MyServer -a ac!@#$12 -v vc123!@#
 * </pre> <b>Log4J Initialization</b><p>For Log4J initialization, the Log4J config file 'log4JConfig.xml' is expected to
 * be in the classpath location:
 * <ul>
 * <li>props/log4jConfig.xml
 * </ul> * @author Application Modernization - Foundations Team
 * @version 1.0.0.193
 */
public final class VistaLinkRpcConsole {

	// static values
	private static final String CRLF = "\r\n";
	private static final int TIMEOUT = 300;
	private static final String RPCCONTEXT = "XOBV VISTALINK TESTER";

	// Defaults, can be overriden from the command line
	private String cfgName = "DemoServer";
	private String accessCode = "joe.123";
	private String verifyCode = "ebony.234";
	private String division = "";
	private String numOfIterations = "1";
	private String useProprietaryMessageFormat = "true";
	private boolean useProprietaryMessageFormatBoolean = true;

	// Log4J setup
	private static final Logger LOGGER = Logger.getLogger(VistaLinkRpcConsole.class);
	private static final Logger LOGGER1 = Logger.getLogger(VistaLinkRpcConsole.class.getName() + ".Other");

	/**
	 * Method main.
	 * @param args see the online help sent to system output when launched
	 */
	public static void main(String[] args) {

		DOMConfigurator.configure("props/log4jConfig.xml");

		// If using programmatic log4j initialization,
		// use following command line parameter to prevent unexpected results
		// as log4j tryes to load config files by default.
		// -Dlog4j.defaultInitOverride=true

		LOGGER.debug("Starting to run samples...");
		VistaLinkRpcConsole app = new VistaLinkRpcConsole(args);
	}

	/**
	 * @param args Passed in arguments ; none used at this time
	 */
	VistaLinkRpcConsole(String[] args) {

		doCommandLineHelp();
		try {
			parseArgs(args);
		} catch (Exception e) {
			LOGGER.debug("Error: " + e.getMessage());
			System.exit(-1);
		}

		useProprietaryMessageFormatBoolean = Boolean.valueOf(useProprietaryMessageFormat).booleanValue();

		RpcRequest vReq = null;
		RpcResponse vResp = null;

		VistaKernelPrincipalImpl myPrincipal = null;
		LoginContext lc = null;

		LOGGER.debug("=====================================");
		LOGGER.debug("Configuration name: " + cfgName);
		LOGGER.debug("=====================================");

		int exitFlag = 0;
		
		try {
			// create the callbackhandler for JAAS login
			CallbackHandlerUnitTest cbhSilentSimple = new CallbackHandlerUnitTest(accessCode, verifyCode, division);

			// create the JAAS LoginContext for login
			lc = new LoginContext(cfgName, cbhSilentSimple);

			// login to server
			lc.login();

			// get JAAS principal after login
			myPrincipal = VistaKernelPrincipalImpl.getKernelPrincipal(lc.getSubject());
			LOGGER.debug(myPrincipal);

			// get connection from JAAS principal
			VistaLinkConnection myConnection = myPrincipal.getAuthenticatedConnection();

			int iCount = Integer.parseInt(numOfIterations);

			for (int i = 0; i < iCount; i++) {
				LOGGER.debug("--------------------------------------------------");
				LOGGER.debug("--------------------------------------------------");
				LOGGER.debug("Starting iteration " + i);

				performTestIteration(myConnection);

				LOGGER.debug("Ending iteration " + i);
				LOGGER.debug("--------------------------------------------------");
				LOGGER.debug("--------------------------------------------------");
			}

		} catch (VistaLoginModuleLoginsDisabledException e) {
			LOGGER1.error("Logins are disabled on the M system.");
			exitFlag = 1;
		} catch (VistaLoginModuleNoJobSlotsAvailableException e) {
			LOGGER1.error("No job slots are available on the M system.");
			exitFlag = 1;
		} catch (VistaLoginModuleUserCancelledException e) {
			LOGGER1.error("User cancelled Login.");
			exitFlag = 1;
		} catch (VistaLoginModuleUserTimedOutException e) {
			LOGGER1.error("User timed out.");
			exitFlag = 1;
		} catch (VistaLoginModuleNoPathToListenerException e) {
			LOGGER1.error("No path found to specified listener.");
			exitFlag = 1;
		} catch (VistaLoginModuleTooManyInvalidAttemptsException e) {
			LOGGER1.error("Login cancelled due to too many invalid login attempts.");
			exitFlag = 1;
		} catch (VistaLoginModuleException e) {
			LOGGER1.error(ExceptionUtils.getFullStackTrace(e));
			exitFlag = 1;
		} catch (LoginException e) {
			LOGGER1.error(ExceptionUtils.getFullStackTrace(e));
			exitFlag = 1;
		} catch (SecurityException e) {
			LOGGER1.error(e.getMessage());
			exitFlag = 1;
		} catch (Exception e) {
			LOGGER1.error(ExceptionUtils.getFullStackTrace(e));
			exitFlag = 1;
		} finally {

			try {
				lc.logout();
			} catch (LoginException e1) {
				LOGGER1.error(ExceptionUtils.getFullStackTrace(e1));
			} finally {
				System.exit(exitFlag);
			}

		}

	}

	private void performTestIteration(VistaLinkConnection myConnection) throws FoundationsException {

		RpcRequest vReq = null;
		RpcResponse vResp = null;

		// do RPC tests

		// ---------------------------------------------------------------------------------------
		// force RPC name fault test
		try {
			vReq = RpcRequestFactory.getRpcRequest(RPCCONTEXT, "XOBV TEST PINGx");
			//			vReq.setUseProprietaryMessageFormat(useProprietaryMessageFormatBoolean);

			vResp = myConnection.executeRPC(vReq);
			displayInfo(myConnection, vReq, vResp);
		} catch (VistaLinkFaultException e) {
			LOGGER1.error(ExceptionUtils.getFullStackTrace(e));
		} catch (FoundationsException e) {
			LOGGER1.error(ExceptionUtils.getFullStackTrace(e));
		}

		// ---------------------------------------------------------------------------------------
		// force RPC Context fault test
		try {
			vReq = RpcRequestFactory.getRpcRequest(RPCCONTEXT + "x", "XOBV TEST PING");
			//			vReq.setUseProprietaryMessageFormat(useProprietaryMessageFormatBoolean);

			vResp = myConnection.executeRPC(vReq);
			displayInfo(myConnection, vReq, vResp);

		} catch (VistaLinkFaultException e) {
			LOGGER1.error(ExceptionUtils.getFullStackTrace(e));
		} catch (FoundationsException e) {
			LOGGER1.error(ExceptionUtils.getFullStackTrace(e));
		}

		// ---------------------------------------------------------------------------------------
		// force RPC name not in RPC Contexrt fault test
		try {
			vReq = RpcRequestFactory.getRpcRequest();
			vReq.setRpcContext(RPCCONTEXT);
			//			vReq.setUseProprietaryMessageFormat(useProprietaryMessageFormatBoolean);

			vReq.setRpcName("XOBV TEST NOT IN CONTEXT");
			vResp = myConnection.executeRPC(vReq);
			displayInfo(myConnection, vReq, vResp);

		} catch (VistaLinkFaultException e) {
			LOGGER1.error(ExceptionUtils.getFullStackTrace(e));
		} catch (FoundationsException e) {
			LOGGER1.error(ExceptionUtils.getFullStackTrace(e));
		}

		// ---------------------------------------------------------------------------------------
		// setup request for remainder of calls
		vReq = RpcRequestFactory.getRpcRequest();
		vReq.setUseProprietaryMessageFormat(useProprietaryMessageFormatBoolean);
		vReq.setRpcContext(RPCCONTEXT);
		vReq.setRpcClientTimeOut(TIMEOUT);

		// ---------------------------------------------------------------------------------------
		// do ping test
		vReq.setRpcName("XOBV TEST PING");

		vReq.clearParams();

		vResp = myConnection.executeRPC(vReq);
		displayInfo(myConnection, vReq, vResp);

		// ---------------------------------------------------------------------------------------
		// do word processing test
		vReq.setRpcName("XOBV TEST WORD PROCESSING");
		vReq.clearParams();
		vResp = myConnection.executeRPC(vReq);
		displayInfo(myConnection, vReq, vResp);

		// ---------------------------------------------------------------------------------------
		// do string test
		vReq.setRpcName("XOBV TEST STRING");

		vReq.clearParams();
		vReq.getParams().setParam(1, "string", "This is a test string!");

		vResp = myConnection.executeRPC(vReq);
		displayInfo(myConnection, vReq, vResp);

		// ---------------------------------------------------------------------------------------
		// do global node results test
		vReq.setRpcName("XOBV TEST GLOBAL NODE");

		vReq.clearParams();
		vReq.getParams().setParam(1, "string", "Hello World");

		vResp = myConnection.executeRPC(vReq);
		displayInfo(myConnection, vReq, vResp);

		// ---------------------------------------------------------------------------------------
		// do RPC list results test
		vReq.setRpcName("XOBV TEST RPC LIST");

		vReq.clearParams();
		vReq.getParams().setParam(1, "string", "XOB");

		vResp = myConnection.executeRPC(vReq);
		displayInfo(myConnection, vReq, vResp);

		// ---------------------------------------------------------------------------------------
		// do two parameter test using setParams()
		vReq.setRpcName("XWB EGCHO SORT LIST");

		vReq.clearParams();
		ArrayList params = new ArrayList();

		ArrayList nums = new ArrayList();
		nums.add("3");
		nums.add("5");
		nums.add("4");
		nums.add("1");
		nums.add("7");
		nums.add("8");
		nums.add("2");
		nums.add("6");
		nums.add("9");

		params.add("LO");
		params.add(nums);
		vReq.setParams(params);

		vResp = (RpcResponse) myConnection.executeRPC(vReq);
		displayInfo(myConnection, vReq, vResp);

		// ---------------------------------------------------------------------------------------
		// do reference type test NOT USING VistaRPCReferenceType and setParams()
		vReq.setRpcName("XWB GET VARIABLE VALUE");

		vReq.clearParams();
		vReq.getParams().setParam(1, "ref", "DT");

		vResp = myConnection.executeRPC(vReq);
		displayInfo(myConnection, vReq, vResp);

		// ---------------------------------------------------------------------------------------
		// do reference type test USING VistaRPCReferenceType and setParams()
		vReq.setRpcName("XWB GET VARIABLE VALUE");

		vReq.clearParams();
		params.clear();

		params.add(new RpcReferenceType("DTIME"));
		vReq.setParams(params);

		vResp = myConnection.executeRPC(vReq);
		displayInfo(myConnection, vReq, vResp);

		// ---------------------------------------------------------------------------------------
		// do xml test
		vReq.setRpcName("XOBV TEST XML RESULTS");

		vReq.clearParams();

		// -- temporary until there is an XML return type add to REMOTE PROCEDURE file
		vReq.setXmlResponse(true);

		vResp = myConnection.executeRPC(vReq);
		displayInfo(myConnection, vReq, vResp);
		vReq.setXmlResponse(false);

		// ---------------------------------------------------------------------------------------
		// do local array test
		vReq.setRpcName("XOBV TEST LOCAL ARRAY");

		vReq.clearParams();
		HashMap hm = new HashMap();
		hm.put("1", "Apple");
		hm.put("2", "Orange");
		hm.put("3", "Pear");
		hm.put("4", "'nana");
		vReq.getParams().setParam(1, "array", hm);

		vResp = myConnection.executeRPC(vReq);
		displayInfo(myConnection, vReq, vResp);

		// ---------------------------------------------------------------------------------------
		// do global array results test
		vReq.setRpcName("XOBV TEST GLOBAL ARRAY");

		vReq.clearParams();
		hm.clear();
		hm.put("1", "CD");
		hm.put("2", "Tape");
		hm.put("3", "DVD");
		vReq.getParams().setParam(1, "array", hm);

		vResp = myConnection.executeRPC(vReq);
		displayInfo(myConnection, vReq, vResp);

		// ---------------------------------------------------------------------------------------
		// big array parameter test
		vReq.setRpcName("XOBV TEST LOCAL ARRAY");

		vReq.clearParams();
		TreeMap tm = new TreeMap();
		tm.put("1", "<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		tm.put("2", "<configTree");
		tm.put("3", "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"");
		tm.put(
			"4",
			"xsi:noNamespaceSchemaLocation=\"C:\\reeng\\main\\modules\\config\\src\\gov\\va\\med\\hds\\config\\xml\\configTree.xsd\">");
		tm.put("5", "<moduleParameterCategory>");
		tm.put("6", "<id>1</id>");
		tm.put("7", "<name>Dashboard</name>");
		tm.put("8", "<preferencePage>gov.va.med.hds.dash.DBPrefPage</preferencePage>");
		tm.put("9", "<parameters>");
		tm.put("10", "<parameter>");
		tm.put("11", "<name>Look and Feel</name>");
		tm.put("12", "<key>Look and Feel^5000;DIC(4,^201^1</key>");
		tm.put("13", "<value>Motif</value>");
		tm.put("14", "</parameter>");
		tm.put("15", "<parameter>");
		tm.put("16", "<name>Selection Mode</name>");
		tm.put("17", "<key>Selection Mode^0^4749^1</key>");
		tm.put("18", "<value>Table-Navigator-Viewer</value>");
		tm.put("19", "</parameter>");
		tm.put("20", "<parameter>");
		tm.put("21", "<name>Window Mode</name>");
		tm.put("22", "<key>Window Mode^0^4750^1</key>");
		tm.put("23", "<value>Single Document Interface</value>");
		tm.put("24", "</parameter>");
		tm.put("25", "</parameters>");
		tm.put("26", "<children>");
		tm.put("27", "<parameterCategory>");
		tm.put("28", "<id>2</id>");
		tm.put("29", "<name>Patient Selection</name>");
		tm.put("30", "<preferencePage>gov.va.med.hds.dash.PtSelPrefPage</preferencePage>");
		tm.put("31", "<parameters>");
		tm.put("32", "<parameter>");
		tm.put("33", "<name>Patient Selection Mode</name>");
		tm.put("34", "<key>Patient Selection Mode^0^4751^1</key>");
		tm.put("35", "<value>single click</value>");
		tm.put("36", "</parameter>");
		tm.put("37", "<parameter>");
		tm.put("38", "<name>Default Patient List</name>");
		tm.put("39", "<key>Default Patient List^375;DIC(9.4,^4752^1</key>");
		tm.put("40", "<value>Patient List A</value>");
		tm.put("41", "</parameter>");
		tm.put("42", "</parameters>");
		tm.put("43", "<children>");
		tm.put("44", "</children>");
		tm.put("45", "</parameterCategory>");
		tm.put("46", "<parameterCategory>");
		tm.put("47", "<id>3</id>");
		tm.put("48", "<name>Lab Collection Times</name>");
		tm.put("49", "<preferencePage>gov.va.med.hds.dash.LabPrefPage</preferencePage>");
		tm.put("50", "<parameters>");
		tm.put("51", "<parameter>");
		tm.put("52", "<name>Lab Collection Times</name>");
		tm.put("53", "<key>Lab Collection Times^5000;DIC(4,^21^28800</key>");
		tm.put("54", "<value>0810</value>");
		tm.put("55", "</parameter>");
		tm.put("56", "<parameter>");
		tm.put("57", "<name>Lab Collection Times</name>");
		tm.put("58", "<key>Lab Collection Times^5000;DIC(4,^21^36000</key>");
		tm.put("59", "<value>1030</value>");
		tm.put("60", "</parameter>");
		tm.put("61", "<parameter>");
		tm.put("62", "<name>Lab Collection Times</name>");
		tm.put("63", "<key>Lab Collection Times^5000;DIC(4,^21^39600</key>");
		tm.put("64", "<value>1130</value>");
		tm.put("65", "</parameter>");
		tm.put("66", "<parameter>");
		tm.put("67", "<name>Lab Collection Times</name>");
		tm.put("68", "<key>Lab Collection Times^5000;DIC(4,^21^66600</key>");
		tm.put("69", "<value>1900</value>");
		tm.put("70", "</parameter>");
		tm.put("71", "<parameter>");
		tm.put("72", "<name>Lab Collection Times</name>");
		tm.put("73", "<key>Lab Collection Times^5000;DIC(4,^21^73800</key>");
		tm.put("74", "<value>2100</value>");
		tm.put("75", "</parameter>");
		tm.put("76", "<parameter>");
		tm.put("77", "<name>Lab Collection Times</name>");
		tm.put("78", "<key>Lab Collection Times^5000;DIC(4,^21^75000</key>");
		tm.put("79", "<value>2350</value>");
		tm.put("80", "</parameter>");
		tm.put("81", "</parameters>");
		tm.put("82", "<children>");
		tm.put("83", "</children>");
		tm.put("84", "</parameterCategory>");
		tm.put("85", "</children>");
		tm.put("86", "</moduleParameterCategory>");
		tm.put("87", "</configTree>");

		params.clear();
		params.add(tm);
		vReq.setParams(params);

		vResp = myConnection.executeRPC(vReq);

		// process array a line at a time
		LOGGER.debug("Display array using java.util.StringTokenizer...");
		StringTokenizer st = new StringTokenizer(vResp.getResults(), "\n");

		int cnt = st.countTokens();
		for (int i = 0; i < cnt; i++) {
			LOGGER.debug("Line " + i + ": " + st.nextToken());
		}

		LOGGER.debug("==================================================");
		// ---------------------------------------------------------------------------------------

	}

	/**
	 * Method displayInfo.
	 * @param vReq Request instance
	 * @param vResp Response instance
	 */
	private void displayInfo(VistaLinkConnection connection, RpcRequest vReq, RpcResponse vResp) {
		try {
			String results = "";

			//server info available for read only/debugging purposes
			VistaLinkServerInfo si = connection.getConnectionInfo();

			// if XML response show node name
			if (vReq.isXmlResponse()) {
				results =
					"<XML Results: Document Elememt Name: "
						+ vResp.getResultsDocument().getDocumentElement().getNodeName()
						+ ">";
			} else {
				results = vResp.getResults();
			}

			// get printable verions of document
			LOGGER.debug("==================================================");
			LOGGER.debug("IP Address: " + si.getAddress().getHostAddress());
			LOGGER.debug("Port: " + si.getPort());
			LOGGER.debug("--------------------------------------------------");
			LOGGER.debug("           RPC Name: " + vReq.getRpcName());
			LOGGER.debug("        RPC Context: " + vReq.getRpcContext());
			LOGGER.debug("RPC Client Time Out: " + vReq.getRpcClientTimeOut());
			//			LOGGER.debug("          Has Error: " + vResp.hasError());
			LOGGER.debug("        Result Type: " + vResp.getResultsType());
			LOGGER.debug("            Results:" + CRLF + results);
			LOGGER.debug("    Raw XML Results:" + CRLF + vResp.getRawResponse());
			LOGGER.debug("==================================================");
		} catch (Exception err) {
			LOGGER.debug("Results Processing Error: " + err.toString() + CRLF + "Class: " + err.getClass().getName());
		}
	}

	/**
	 * Method handleResponseException.
	 * @param ex Exception instance
	 */
	//	private static void handleResponseException(VistaResponseException ex) {
	//		LOGGER1.error("==================================================");
	//		LOGGER1.error(
	//			CRLF + "Error Code: " + ex.getCode() + CRLF + "   Message: " + ex.getMessage());
	//	}

	//	private static void handleResponseException(Exception e) {
	//		LOGGER1.error("==================================================");
	//		LOGGER1.error(ExceptionUtils.getFullStackTrace(e));
	//	}

	/**
	 * parses the command-line arguments
	 * @param args
	 */
	private void parseArgs(String[] args) throws Exception {

		int i = 0;
		if (args.length < 1) {

			// if no args passed, always show help so defaults being used are known.
			throw new Exception("Expected parameters, did not recieve any.");

		} else {

			do {
				// first do checks
				if (args[i].substring(0, 1).equals("-")) {
					if (i + 1 < args.length) {
						if (((String) args[i + 1]).substring(0, 1).equals("-")) {
							String errStr =
								"Argument Error: '"
									+ args[i]
									+ "' command line parameter cannot be followed by a flag ("
									+ args[i
									+ 1]
									+ ")";
							LOGGER.error(errStr);
							throw new Exception(errStr);
						}
					} else {
						String errStr =
							"Argument Error: '" + args[i] + "' command line parameter must be followed by a value";
						LOGGER.error(errStr);
						throw new Exception(errStr);
					}
				} else {

					String errStr =
						"Argument Error: '" + args[i] + "' command line parameter expected to be a -flag parameter.";
					LOGGER.error(errStr);
					throw new Exception(errStr);

				}

				// passed checks
				if (args[i].equals("-s")) {
					cfgName = args[++i];
				} else if (args[i].equals("-a")) {
					accessCode = args[++i];
				} else if (args[i].equals("-v")) {
					verifyCode = args[++i];
				} else if (args[i].equals("-d")) {
					division = args[++i];
				} else if (args[i].equals("-i")) {
					numOfIterations = args[++i];
				} else if (args[i].equals("-p")) {
					useProprietaryMessageFormat = args[++i];
				} else {
					String errStr = "Argument Error: '" + args[i] + "' is an unsupported flag.";
					LOGGER.error(errStr);
					throw new Exception(errStr);
				}

				i++;

			} while (i < args.length);
		}

	}

	/**
	 * Prints help on command-line arguments needed for this app to the console.
	 */
	void doCommandLineHelp() {

		System.out.println("\nThis application defaults to using the following connection values:\n");
		System.out.println("    Server appname: " + cfgName);
		System.out.println("    Access code: " + accessCode);
		System.out.println("    Verify code: " + verifyCode);
		System.out.println("    Division:  " + division);
		System.out.println("    Number of iterations:  " + numOfIterations);
		System.out.println("    Use proprietary message format instead of XML:  " + useProprietaryMessageFormat);
		System.out.println("\n Override defaults with the following optional flags:\n");
		System.out.println("    -s server alias to use from the JAAS config file");
		System.out.println("    -a access code");
		System.out.println("    -v verify code");
		System.out.println("    -d division ien");
		System.out.println("    -i number of iterations");
		System.out.println("    -p {true|false} use proprietary message format instead of XML");
		System.out.println(
			"\ne.g.: java -Djava.security.auth.login.config=./jaas.config VistaLinkRpcConsole -s MyServer -a ac!@#$12 -v vc123!@#");
		System.out.println(
			"\nAlso, for Log4J initialization, the Log4J config file 'log4JConfig.xml' is expected to be in the classpath location");
		System.out.println("props/log4jConfig.xml\n");
	}
}
